/*
 * Copyright (C) 2002,2003 Daniel Heck
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: editor.cc,v 1.2 2003/01/19 17:58:33 dheck Exp $
 */
#include "editor.hh"
#include "world.hh"
#include "lua.hh"
#include "px/sdl.hh"
#include <iostream>

using namespace std;
using namespace enigma;
using namespace enigma::editor;

namespace
{
    class Action {
    public:
        void perform() { on_perform(); }
        void undo() { on_undo(); }
    private:
        virtual void on_perform() = 0;
        virtual void on_undo() = 0;
    };

    class Creator {
    public:
        virtual ~Creator() {}
        virtual void emit_code(ostream &os) = 0;
    };

    class StoneCreator : public Creator {
    public:
    private:
        void emit_code(ostream &os) {
            os << "set_stone(\"" << kind 
               << "\"," << pos.x << ","<<pos.y<<")\n";
        }
        string kind;
        GridPos pos;
    };
}


/* 
 * This is the class that is responsible for displaying the level
 * currently being edited.
 */
   
namespace
{
    struct Point 
    {
        // Constructors.
        Point (double a, double b) : x(a), y(b) {}

        // Variables.
        double x,y;             // Coordinates of the point.
    };

    struct Line 
    {
        // Constructors.
        Line (Point from, Point to) : p(from), q(to) {}
        Line (double x1, double y1, double x2, double y2)
            : p(x1,y1), q(x2, y2)
        {}

        // Variables.
        Point p, q;             // Start- and endpoint of the line.
    };


    class Image;

    class ImagePack {
    public:
        static ImagePack *make_pack (int size);

        union {
            struct {
                int layerbits : 5; // Which layers are present?
                
            } pack_info;
            ImagePack *next_avail;
        };
        Image *images[0];
    };

    class LevelView 
    {
    public:
        // Constructors.
        LevelView (int w, int h);

        // Functions.
        void set_image (int x, int y, int layer, Image *img);

        int add_line (const Line &l);
        void remove_line (int linetag);
    private:
        // Private functions.

        // Variables.
        int width, height;      // Width and height of the level.
    };
}

LevelView::LevelView (int w, int h)
    : width (w), height (h)
{
}

void 
LevelView::set_image (int x, int y, int layer, Image *img)
{
}


//======================================================================
// THE EDITOR
//======================================================================
namespace
{
    class Editor : sdl::EventHandler {
    public:
        Editor();
        ~Editor();

        void run();
    private:
        // Private methods.

        // EventHandler interface.
        bool on_mousemotion(SDL_Event &e);
        bool on_mousedown(SDL_Event &e);
        bool on_keydown(SDL_Event &e);
        
        // Variables.
    };
}

Editor::Editor()
{}

Editor::~Editor()
{}

void
Editor::run()
{
}

bool Editor::on_mousemotion(SDL_Event &e)
{
    return false;
}

bool Editor::on_mousedown(SDL_Event &e)
{
    return false;
}

bool Editor::on_keydown(SDL_Event &e)
{
    return false;
}



void
editor::Run()
{
    lua::Dofile("editor.lua");
    Editor e;
    e.run();
}
