/*
 * Copyright (C) 2002,2003 Daniel Heck
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: game.hh,v 1.3 2003/01/30 20:33:33 dheck Exp $
 */
#ifndef ENIGMA_GAME_HH
#define ENIGMA_GAME_HH

#include "enigma.hh"

//----------------------------------------
// Data types
//----------------------------------------
namespace enigma
{
/*
** Enigma can run its own levels but also emulate various versions of
** Oxyd.  All these games behave similarly, but there are also quite a
** few differences in object behaviour, visual appearance, etc.  The
** game type is set at program startup and cannot be changed during
** the game.
*/
    enum GameType {
        GAMET_FIRST,

        GAMET_ENIGMA = GAMET_FIRST,
        GAMET_OXYD1,
        GAMET_PEROXYD,
        GAMET_OXYDEXTRA,
        GAMET_OXYDMAGNUM,

        GAMET_LAST = GAMET_OXYDMAGNUM,
        GAMET_COUNT
    };

/*
** This datastructure contains information about single levels
*/
    struct LevelInfo 
    {
        LevelInfo(const string &fn, const string &n, const string &a, int par_time=99*60+59)
            : filename(fn), name(n), author(a), best_time(par_time)
        {}

        // Variables.
        string filename;        // Filename of the level _without_ extension
        string name;            // Complete name of the level
        string author;          // Author of the level
        int best_time;          // Best time in seconds
    };

    class LevelPack {
    public:
        LevelPack(const string &initfile, const string &n)
            : init_file(initfile),name(n)
        {}

        // Load init_file to initialize the level list.
        void init();

        // Variables.
        string init_file;       // Text file containing the level list
        string name;            // Name of the level pack ("enigma", "oxyd", ...)
        vector<LevelInfo> levels;
    };
}

//----------------------------------------
// Interface to game engine
//----------------------------------------
namespace enigma
{
    enum {
        DIFFICULTY_EASY = 1,
        DIFFICULTY_HARD
    };

    extern vector<LevelPack *> LevelPacks;
    extern unsigned            CurrentLevelPack;
    extern bool ConserveLevel;  // True: do not reset level when player dies
    extern int Difficulty;      // Current difficulty level (1=easy,2=hard)

    void AddLevelPack (const char *init_file, const char *name);
    void StartGame (LevelPack *lp, unsigned levelidx);
    void FinishLevel();
    void RestartLevel();
    void RestartGame();
    void QuitGame();
}
#endif
