/*
 * Copyright (C) 2002 Daniel Heck
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
#ifndef OPTIONS_HH
#define OPTIONS_HH

#include <string>

namespace options
{
    using std::string;
    
    class OptionObserver {
    public:
        virtual ~OptionObserver() {}
        virtual void option_changed (const std::string &name) = 0;
    };

    extern int    BitsPerPixel;
    extern double MouseSpeed;
    extern double FrictionFactor;
    extern bool   WizardMode;
    extern bool   FullScreen;
    extern bool   ShowFPS;
    extern bool   UseAlpha;
    extern bool   Nograb;
    extern double SoundVolume;
    extern double MusicVolume;
    extern double StereoSeparation;

    struct LevelStatus {
        LevelStatus(int easy=-1, int hard=-1, int fin=0)
            : par_easy(easy),
              par_hard(hard),
              finished(fin)
        {}
        int par_easy, par_hard; // Best time in seconds (-1: use level default)
        int finished;           // User completed this level?
                                // 0=no,1=easy,2=hard,3=easy&hard
    };

    /* Get the status of a particular level in a particular pack.
       Returns 0 if no record for this level exists. */
    LevelStatus *GetLevelStatus(const string &levelpack,
                                const string &levelname);

    /* Set the status of a particular level.  The previous entry (if
       available) for this level is discarded. */
    void SetLevelStatus(const string &levelpack,
                        const string &levelname,
                        const LevelStatus &stat);

    void SetLevelTime(const string &pack, const string &level, int difficulty, int time);
    void SetLevelFinished(const string &pack, const string &level, int difficulty);
    
    /* Save the current set of options to the user's configuration
       file. (Either ~/.enigmarc on Unix systems or ./enigmarc.lua on
       Win32. */
    bool Save ();

    /* Try to load the user's configuration file.  Returns true if
       successful. */
    bool Load ();
}


#endif
