/*
 * Copyright (C) 2002,2003 Daniel Heck
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: player.hh,v 1.2 2003/01/30 20:33:33 dheck Exp $
 */
#ifndef PLAYER_HH
#define PLAYER_HH

#include "fwd.hh"
#include "world.hh"
#include <vector>

namespace player
{
    using world::Item;
    using world::Actor;

    class Inventory {
    public:
        Inventory();
        ~Inventory();
        void add_item(Item *i);
        int size() { return items.size(); } // number of items
        void clear();
	
        bool is_full() const { return items.size() == max_items; }
	
        void rotate();
        void activate_first();
        Item *get_item(int idx);
        Item *yield_item(int idx);
        Item *yield_first();
    private:
        // Private methods.
        void redraw();

        // Private variables.
        static const unsigned max_items;
        std::vector<Item*> items;
    };

    // Start a new game for N players.  Every player begins with two
    // extra lifes.
    void NewGame(int nplayers);

    /* This is called whenever a new level is reached in a running
       game.  The inventories of all players are cleaned up, i.e., all
       items except for extra lifes are removed. */
    void NewWorld();


    /* Call this function as soon as the current level is finished; it
       will remove the actors of all players from the level. */
    void LevelFinished();
    
    /* This function is only used by the YinYang items to exchange the
       two players. */
    void SwapPlayers();

    void SetCurrentPlayer(unsigned iplayer);

    
    Inventory *GetInventory(int iplayer);
    int CurrentPlayer();
    void Suicide();

    void AddActor(int iplayer, Actor *a);

    bool AllActorsDead();

    void InhibitPickup(bool fl);
    void PickupItem(Actor *a, world::GridPos p);
    void RotateInventory();
    void ActivateItem();

    void Tick (double dtime);
}
#endif
